@@ -18,33 +18,46 @@ from django.contrib.auth.models import User
 
 
 class LDAPBackend:
-  def authenticate(self,username=None,password=None):
+  def authenticate(self, username=None, password=None):
+    if settings.LDAP_USER_DN_TEMPLATE is not None:
+      settings.LDAP_BASE_USER = settings.LDAP_USER_DN_TEMPLATE % {'username': username}
+      settings.LDAP_BASE_PASS = password
     try:
-      conn = ldap.open(settings.LDAP_SERVER, port=settings.LDAP_PORT)
+      conn = ldap.initialize(settings.LDAP_URI)
       conn.protocol_version = ldap.VERSION3
+      if settings.LDAP_USE_TLS:
+        conn.start_tls_s()
       conn.simple_bind_s( settings.LDAP_BASE_USER, settings.LDAP_BASE_PASS )
     except ldap.LDAPError:
       traceback.print_exc()
       return None
+
     scope = ldap.SCOPE_SUBTREE
     filter = settings.LDAP_USER_QUERY % username
     returnFields = ['dn','mail']
     try:
       resultID = conn.search( settings.LDAP_SEARCH_BASE, scope, filter, returnFields )
       resultType, resultData = conn.result( resultID, 0 )
-      if len(resultData) != 1: #User does not exist
+      if len(resultData) != 1: # User does not exist
         return None
+
       userDN = resultData[0][0]
-      try: userMail = resultData[0][1]['mail'][0]
-      except: userMail = "Unknown"
+      try:
+        userMail = resultData[0][1]['mail'][0].decode("utf-8")
+      except Exception:
+        userMail = "Unknown"
+
       conn.simple_bind_s(userDN,password)
       try:
         user = User.objects.get(username=username)
-      except: #First time login, not in django's database
-        randomPasswd = User.objects.make_random_password(length=16) #To prevent login from django db user
+      except Exception:  # First time login, not in django's database
+        # To prevent login from django db user
+        randomPasswd = User.objects.make_random_password(length=16)
         user = User.objects.create_user(username, userMail, randomPasswd)
         user.save()
+
       return user
+
     except ldap.INVALID_CREDENTIALS:
       traceback.print_exc()
       return None
