@@ -15,14 +15,14 @@ limitations under the License."""
 import os
 from smtplib import SMTP
 from socket import gethostname
-from email.MIMEMultipart import MIMEMultipart
-from email.MIMEText import MIMEText
-from email.MIMEImage import MIMEImage
-from httplib import HTTPConnection
-from urlparse import urlsplit
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
+from email.mime.image import MIMEImage
+from six.moves.http_client import HTTPConnection
+from six.moves.urllib.parse import urlsplit
 from time import ctime, strftime
 from traceback import format_exc
-from graphite.util import getProfile
+from graphite.user_util import getProfile
 from graphite.logger import log
 from graphite.account.models import MyGraph
 
@@ -31,15 +31,17 @@ from django.http import HttpResponse
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
 
+
 def composer(request):
   profile = getProfile(request)
   context = {
-    'queryString' : request.GET.urlencode(),
+    'queryString' : request.GET.urlencode().replace('+','%20'),
     'showTarget' : request.GET.get('showTarget',''),
     'user' : request.user,
     'profile' : profile,
     'showMyGraphs' : int( profile.user.username != 'default' ),
     'searchEnabled' : int( os.access(settings.INDEX_FILE, os.R_OK) ),
+    'refreshInterval': settings.AUTO_REFRESH_INTERVAL,
     'debug' : settings.DEBUG,
     'jsdebug' : settings.DEBUG,
   }
@@ -70,8 +72,7 @@ def mygraph(request):
       try:
         newGraph = MyGraph(profile=profile,name=graphName,url=url)
         newGraph.save()
-
-      except:
+      except Exception:
         log.exception("Failed to create new MyGraph in /composer/mygraph/, graphName=%s" % graphName)
         return HttpResponse("Failed to save graph %s" % graphName)
 
@@ -99,7 +100,10 @@ def send_email(request):
     if query: path += '?' + query
     conn = HTTPConnection(server)
     conn.request('GET',path)
-    resp = conn.getresponse()
+    try: # Python 2.7+, use buffering of HTTP responses
+      resp = conn.getresponse(buffering=True)
+    except TypeError:  # Python 2.6 and older
+      resp = conn.getresponse()
     assert resp.status == 200, "Failed HTTP response %s %s" % (resp.status, resp.reason)
     rawData = resp.read()
     conn.close()
@@ -116,5 +120,5 @@ def send_email(request):
     s.sendmail('composer@%s' % gethostname(),recipients,message.as_string())
     s.quit()
     return HttpResponse( "OK" )
-  except:
-    return HttpResponse( format_exc() )
+  except Exception:
+    return HttpResponse(format_exc())
