@@ -1,17 +1,17 @@
-import time
 import os
 
 from django.db import models
-from django.contrib import admin
+from tagging.models import Tag
+
+from graphite.events.compat import ModelTaggedItemManager
 
 if os.environ.get('READTHEDOCS'):
     TagField = lambda *args, **kwargs: None
 else:
     from tagging.fields import TagField
 
-class Event(models.Model):
-    class Admin: pass
 
+class Event(models.Model):
     when = models.DateTimeField()
     what = models.CharField(max_length=255)
     data = models.TextField(blank=True)
@@ -24,8 +24,17 @@ class Event(models.Model):
         return "%s: %s" % (self.when, self.what)
 
     @staticmethod
-    def find_events(time_from=None, time_until=None, tags=None):
-        query = Event.objects.all()
+    def find_events(time_from=None, time_until=None, tags=None, set_operation=None):
+
+        if tags is not None:
+            if set_operation == 'union':
+                query = Event.tagged.with_any(tags)
+            elif set_operation == 'intersection':
+                query = Event.tagged.with_intersection(tags)
+            else:
+                query = Event.tagged.with_all(tags)
+        else:
+            query = Event.objects.all()
 
         if time_from is not None:
             query = query.filter(when__gte=time_from)
@@ -33,10 +42,6 @@ class Event(models.Model):
         if time_until is not None:
             query = query.filter(when__lte=time_until)
 
-        if tags is not None:
-            for tag in tags:
-                query = query.filter(tags__iregex=r'\b%s\b' % tag)
-
         result = list(query.order_by("when"))
         return result
 
@@ -45,8 +50,11 @@ class Event(models.Model):
             when=self.when,
             what=self.what,
             data=self.data,
-            tags=self.tags,
+            tags=self.tags.split(),
             id=self.id,
         )
 
-admin.site.register(Event)
+
+# We use this rather than tagging.register() so that tags can be exposed
+# in the admin UI
+ModelTaggedItemManager().contribute_to_class(Event, 'tagged')
