@@ -12,10 +12,16 @@ You may obtain a copy of the License at
    See the License for the specific language governing permissions and
    limitations under the License."""
 
-from django.conf.urls.defaults import *
+from django.conf.urls import url
+from . import views
 
-urlpatterns = patterns('graphite.metrics.views',
-  ('^find/?$', 'find_view'),
-  ('^expand/?$', 'expand_view'),
-  ('^context/?$', 'context_view'),
-)
+urlpatterns = [
+    url(r'^/index\.json$', views.index_json, name='metrics_index'),
+    url(r'^/find/?$', views.find_view, name='metrics_find'),
+    url(r'^/expand/?$', views.expand_view, name='metrics_expand'),
+    url(r'^/get-metadata/?$', views.get_metadata_view,
+        name='metrics_get_metadata'),
+    url(r'^/set-metadata/?$', views.set_metadata_view,
+        name='metrics_set_metadata'),
+    url(r'^/?$', views.find_view, name='metrics'),
+]
